' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "GLFW.INC"
#INCLUDE ONCE "GDIPUTILS.INC"

FUNCTION PBMAIN () AS LONG

   LOCAL nWidth, nHeight, running, frames, x, y, hr AS LONG
   LOCAL t, t0, fps AS DOUBLE
   LOCAL szTitlestr AS ASCIIZ * 200
   LOCAL hWnd, hDC, hFont, hOldFont, FontBase AS DWORD
   LOCAL rot AS SINGLE
   LOCAL szText AS ASCIIZ * 256
   LOCAL TextureWidth, TextureHeight AS LONG
   LOCAL strTextureData AS STRING
   LOCAL glHandle AS DWORD
   DIM   gmf(0 TO 255) AS GLYPHMETRICSFLOAT

   ' Initialize GLFW
   glfwInit

   ' Open OpenGL window
   IF ISFALSE glfwOpenWindow(640, 480, 0, 0, 0, 0, 16, 0, %GLFW_WINDOW) THEN
      glfwTerminate
      EXIT FUNCTION
   END IF

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("Lights.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   ' Assign an OpenGL handle to this texture
   glGenTextures 1, glHandle

   ' Build linear mipmapped texture
   glBindTexture %GL_TEXTURE_2D, glHandle
   gluBuild2DMipmaps %GL_TEXTURE_2D, 3, TextureWidth, TextureHeight, %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR_MIPMAP_NEAREST
   ' Texturing contour anchored to the Object
   glTexGeni %GL_S, %GL_TEXTURE_GEN_MODE, %GL_OBJECT_LINEAR
   glTexGeni %GL_T, %GL_TEXTURE_GEN_MODE, %GL_OBJECT_LINEAR
   ' Auto texture generation
   glEnable %GL_TEXTURE_GEN_S
   glEnable %GL_TEXTURE_GEN_T

   ' Select smooth shading
   glShadeModel %GL_SMOOTH
   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Specify the clear value for the depth buffer
   glClearDepth 1.0!
   ' Enable depth comparisons and update the depth buffer
   glEnable %GL_DEPTH_TEST
   ' Specify the value used for depth-buffer comparisons
   glDepthFunc %GL_LEQUAL
   ' Really nice perspective calculations
   glHint %GL_PERSPECTIVE_CORRECTION_HINT, %GL_NICEST
   ' Enable default light (quick and dirty)
   glEnable %GL_LIGHT0
   ' Enable lighting (doesn't work with my computer)
'  glEnable %GL_LIGHTING
   ' Enable texture mapping
   glEnable %GL_TEXTURE_2D
   ' Select the texture
   glBindTexture %GL_TEXTURE_2D, glHandle

   ' Storage for 256 characters
   FontBase = glGenLists(256)
   ' Build the font
   hFont = CreateFont(-12, _                             ' Height of the font
                       0, _                              ' Width of the font
                       0, _                              ' Angle of escapement
                       0, _                              ' Orientation angle
                       %FW_BOLD, _                       ' Font weight
                       %FALSE, _                         ' Italic
                       %FALSE, _                         ' Underline
                       %FALSE, _                         ' Strikeout
                       %SYMBOL_CHARSET, _                 ' Character set identifier
                       %OUT_TT_PRECIS, _                 ' Output precision
                       %CLIP_DEFAULT_PRECIS, _           ' Clipping precision
                       %ANTIALIASED_QUALITY, _           ' Output quality
                       %FF_DONTCARE OR %DEFAULT_PITCH, _ ' Family and pitch
                       "Wingdings")                      ' Font name

   ' Retrieve the window handle
   hWnd = FindWindow("GLFW", BYVAL %NULL)
   ' Retrieve the device context
   hDC = GetDC(hWnd)
   ' Select our font
   hOldFont = SelectObject(hDC, hFont)
   ' Creates a set of display lists, one for each glyph of the
   ' currently selected outline font of a device context
   wglUseFontOutlines _
      hDC, _                  ' Select the current DC
      0, _                    ' Starting character
      255, _                  ' Number of display lists to build
      FontBase, _             ' Starting display lists
      0.1!, _                 ' Deviation from the true outlines
      0.2!, _                 ' Font thickness in the Z direction
      %WGL_FONT_POLYGONS, _   ' Use polygons, not lines
      gmf(0)
   ' Select previous font
   SelectObject hDC, hOldFont
   ' Delete the font
   DeleteObject hFont
   ' Release the device context
   ReleaseDC hWnd, hDC

   ' Enable sticky keys
   glfwEnable %GLFW_STICKY_KEYS

   ' Disable vertical sync (on cards that support it)
   glfwSwapInterval 0

   ' Main loop
   running = %TRUE
   frames = 0
   t0 = glfwGetTime

   DO WHILE running

      ' Get time and mouse position
      t = glfwGetTime
      glfwGetMousePos x, y
      ' Calculate and display FPS (frames per second)
      IF t - t0 > 1.0! OR frames = 0 THEN
         fps = frames / (t-t0)
         wsprintf szTitlestr, "NEHE Lesson 15 (%i FPS)", BYVAL fps
         glfwSetWindowTitle szTitlestr
         t0 = t
         frames = 0
      END IF
      frames = frames + 1

      '  Get window size (may be different than the requested size)
      glfwGetWindowSize nWidth, nHeight
      IF nHeight <= 0 THEN nHeight = 1

      ' Set viewport
      glViewport 0, 0, nWidth, nHeight

      ' Clear the screen and depth buffers
      glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

      ' Select and setup the projection matrix
      glMatrixMode %GL_PROJECTION
      glLoadIdentity

      ' Calculate the aspect ratio of the window
      gluPerspective 45.0!, nWidth/nHeight, 1.0!, 100.0!

      ' Select and setup the modelview matrix
      glMatrixMode %GL_MODELVIEW
      glLoadIdentity

      ' Position the text
      glTranslatef 1.1! * COS(rot / 16.0!), 0.8! * SIN(rot / 20.0!), -3.0!
      ' Rotate on the X axis
      glRotatef rot, 1.0!, 0.0!, 0.0!
      ' Rotate on the Y axis
      glRotatef rot * 1.2!, 0.0!, 1.0!, 0.0!
      ' Rotate on the Z axis
      glRotatef rot * 1.4!, 0.0!, 0.0!, 1.0!
      ' Center on X, Y, Z axis
      glTranslatef -0.35!, -0.35!, 0.1!
      ' Draw a skull and a crossbones symbol
      szText = "N"
      ' Pushes the display list bits
      glPushAttrib %GL_LIST_BIT
      ' Sets the base character to 0
      glListBase FontBase
      ' Draws the display list text
      glCallLists LEN(szText), %GL_UNSIGNED_BYTE, szText
      ' Pops the display list bits
      glPopAttrib
      ' Increase the rotation value
      rot = rot + 0.1!

      ' Swap buffers
      glfwSwapBuffers

      ' Check if the ESC key was pressed or the window was closed
      running = NOT glfwGetKey(%GLFW_KEY_ESC) AND glfwGetWindowParam(%GLFW_OPENED)

   LOOP

   ' Deletes the lists
   glDeleteLists FontBase, 256

   ' Delete the texture
   IF glHandle THEN glDeleteTextures(1, glHandle)

   ' Close OpenGL window and terminate GLFW
   glfwTerminate

END FUNCTION
